#include "pyretic.h"

/* Initializes the pointed-at character with the given data.
 */
int init_character(cartesian initloc, const char *imgfile, character *toinit) {
   toinit->location = initloc;
   toinit->tile = to_grid_loc(initloc);
   toinit->image = load_png(imgfile, NULL);
   toinit->state = 1;
   memset(&(toinit->velocity), 0, sizeof(cartesian));
   return 0;
}

/* Takes a cartesian with a literal (x, y) pixel position on the screen,
   returns a cart_int with the corresponding game tile position.
*/
cart_int to_grid_loc(cartesian xy_loc) {
   cart_int retval;
   retval.x = (int) floor((xy_loc.x) / 32);
   retval.y = (int) floor((xy_loc.y) / 32);
   return retval;
}

/* Takes a cart_int with a game tile position, returns a cartesian with 
   the literal (x, y) position in pixels.
*/
cartesian to_xy_loc(cart_int grid_loc) {
   cartesian retval;
   retval.x = grid_loc.x * 32;
   retval.y = grid_loc.y * 32;
   return retval;
}

/* Draws character.image to target at 
   (character.location.x - 16, character.location.y - 16).
*/
int draw_character_n(BITMAP *target, character *todraw) { // this has an extra _n because draw_character() is in Allegro
   masked_blit(todraw->image, target, 0,0, todraw->location.x - 16, todraw->location.y - 16, 32, 32); // draw character
   return 0;
}

/* Like the new operator in C++/Java. Returns a cartesian with 
   the given X and Y.
*/
cartesian make_cartesian(double x, double y) {
  cartesian *retval = malloc(sizeof(cartesian));
  retval->x = x;
  retval->y = y;
  return *retval;
}

/* A bit like new cart_int() in C++/Java. Returns a cart_int with 
   the given x and y. 
*/
cart_int make_cart_int(int x, int y) {
   cart_int retval;
   retval.x = x;
   retval.y = y;
   return retval;
}

/* Updates a character. This code will govern the movement of
   a quiescent character, i.e., one that is not moving of its
   own accord (like controlling the PC with arrow keys). If it
   wants to move on its own, that is handled from the calling
   context. This does: gravity if character is falling, checking
   whether the character is on a platform/crate and governing the
   character.state variable, updating the position from the 
   velocity, and preventing the character from leaving the screen.
   It's basically character physics.
*/

int update_character(character *toupdate, map *theMap) {
   if (toupdate->state)
      toupdate->velocity.y += 0.5; // handle gravity 
   toupdate->location.x += toupdate->velocity.x; // update position
   toupdate->location.y += toupdate->velocity.y;
   if (theMap->platforms[toupdate->tile.x][toupdate->tile.y] == 1 && 
       toupdate->state == 1) { // check if cur. pos. is on a platform
      toupdate->location.y = toupdate->tile.y * 32;
      toupdate->state = 0;
   }
   if (theMap->obj_map[toupdate->tile.x][toupdate->tile.y + 1] == 1 && 
       toupdate->state == 1) { // check if cur. pos. is on a crate
      toupdate->location.y = toupdate->tile.y * 32;
      toupdate->state = 0;
   } 
   if (theMap->platforms[toupdate->tile.x][toupdate->tile.y - 1] == 1 && 
       (int)toupdate->location.y % 32 < 4) { // I don't know why 
      toupdate->velocity.y = 0;             // this code helps, but it does.
   } 
   if (toupdate->location.y < 10) { // Prevent character from leaving area
      toupdate->location.y = 10;
      toupdate->velocity.y = 0;
   }
   if (toupdate->location.x < 10) {
      toupdate->location.x = 10;
      toupdate->velocity.x = 0;
   }
   if (toupdate->location.x > SCREEN_W - 10) {
      toupdate->location.x = SCREEN_W - 10;
      toupdate->velocity.x = 0;
   }
   if ((theMap->platforms[toupdate->tile.x][toupdate->tile.y] == 0) && 
       (theMap->obj_map[toupdate->tile.x][toupdate->tile.y + 1] == 0)) 
      toupdate->state = 1;  // check if character is in midair (not on platform or crate)
   return 0;
}
