#include "pyretic.h"

volatile int speed;

void ticker(void) {
  speed++;
}
END_OF_FUNCTION(ticker)

map *theMap;
BITMAP *buffer;

/* Takes a bitmap, string, an x, y, width and height, and draws a text dialog box
   at the given place */
int draw_dialog(BITMAP *target, int x, int y, const char *string, int width, int height) {
  rectfill(target, x - 5, y - 5, x + width + 5, y + height + 5, BLACK); // draw a box
  hline(target, x, y, x + width, WHITE);
  vline(target, x, y, y + height, WHITE);
  hline(target, x, y + height, x + width, WHITE);
  vline(target, x + width, y, y + height, WHITE);
  int newx = x + 5, newy = y + 5; // track current word's coords
  int newp; // to store index of space
  char *string2 = malloc(500); // storage
  char *stostr; // to store original value of string2
  stostr = string2;
  strcpy(string2, string);
  char *nexttok = malloc(50); // more storage
  while ((newp = strcspn(string2, " "))) { // this uses strcspn() because all other index functions return pointers
    strncpy(nexttok, string2, newp + 1); // copy first word from string to token
    nexttok[newp + 1] = '\0'; // null-terminate
    if ((newx + text_length(font, nexttok)) > (x + width)) {
      newx = x + 5;
      newy += 8;
    }
    textout_ex(target, font, nexttok, newx, newy, WHITE, BLACK); // draw it
    newx += text_length(font, nexttok); // track coordinates
    string2 += newp + 1; // advance text pointer
  }
  free((void *) stostr);
  free((void *) nexttok); // avoid memory leaks
  return 0;
}

int main(int argc, char** argv) {
  int mouseButtons;
  initialize_allegro(0, &mouseButtons, 640, 480);
  LOCK_VARIABLE(speed);
  LOCK_FUNCTION(ticker);
  install_int_ex(ticker, BPS_TO_TIMER(60));
  set_window_title("Psychopyretic");
  theMap = malloc(sizeof(map));
  char *mapfile = malloc(64);
  strcpy(mapfile, "data/maps/mainmap.ppm");
  init_map(theMap);
  if (load_map(theMap, "data/maps/mainmap.ppm") != 0) {
      printf("Map file not found: %s\n", mapfile);
      exit(-1);
  }
  int i,j;
  buffer = create_bitmap(640, 480);
  BITMAP *dungeon = load_png("data/images/dungeon.png", NULL);
  BITMAP *crateimg = load_png("data/images/crate-map.png", NULL);
  BITMAP *platmap = load_png("data/images/platmap.png", NULL);
  BITMAP *pointer = load_png("data/images/pointer.png", NULL);
  BITMAP *explosions = load_png("data/images/explosions.png", NULL);
  cartesian initloc = make_cartesian(SCREEN_W/2, SCREEN_H/2);
  init_character(initloc, "data/images/character.png", &(theMap->pc));
  npc tnpc;
  load_npc(to_xy_loc(make_cart_int(14,14)), "data/images/npc1.png", &tnpc);
  tnpc.cdata.location.x += 16;
  int numJumps = 0;
  int isAlive = 1;
  (theMap->pc).state = 1;
  int pyropower = 0;
  int dbg_field = 0;
  int ex_frame = 41, ex_x = 0, ex_y = 0; // to display explosion graphic
  while (!key[KEY_ESC]) {
    while (speed > 0) {
      /* start logic-update routine */
      pyropower++; // Handle psychopyretic power
      if (pyropower >= 255) // Can't be over 255
	pyropower = 255;
      if ((mouse_b & 1) && pyropower >= 255) { // Click to use it
	ex_frame = 0;
	ex_x = (int) floor(mouse_x / 32);
	ex_y = (int) floor((mouse_y + 16) / 32);
	damage_crate(theMap, lookup_crate(theMap, ex_x, ex_y), 2);
	pyropower = 0;
      }
      (theMap->pc).tile = to_grid_loc((theMap->pc).location); // Update tile position
      if (key[KEY_UP] && isAlive) { // Handle jumping
	if (!numJumps) {
	  (theMap->pc).velocity.y = -10;
	  numJumps = 1;
	}
      }
      if (mouse_b & 2) { // debug--able to create crates
	create_crate(theMap, (int) floor(mouse_x / 32), (int) floor(mouse_y / 32));
      }
      /* if (key[KEY_DOWN]) {
	 velocity.len -= 0.1;
	 } */
      /* velocity.len *= 0.999; */
      for (i = 0; i < 300; i++) { // update speed/location of crates
	if (theMap->crates[i].used) {
	  update_crate(theMap, i);
	}
      }
      if (key[KEY_LEFT] && isAlive) { // handle moving PC with left/right arrow keys
	if (theMap->obj_map[(theMap->pc).tile.x - 1][(theMap->pc).tile.y] == 0 || (int)(theMap->pc).location.x % 32 >= 10) {
	  (theMap->pc).location.x += -2;
	}
      }
      if (key[KEY_RIGHT] && isAlive) {
	if (theMap->obj_map[(theMap->pc).tile.x + 1][(theMap->pc).tile.y] == 0 || (int)(theMap->pc).location.x % 32 <= 24) {
	  (theMap->pc).location.x += 2;
	}
      }
      if (isAlive) { // update speed/location of PC
	update_character(&(theMap->pc), theMap);
      }
      if ((theMap->pc).location.y > SCREEN_H + 10) { // PC dies if he falls off the bottom
	isAlive = 0;
      }
      if (!(theMap->pc).state) {
	(theMap->pc).velocity.y = 0;
	numJumps = 0;
      }
      if (!isAlive) {
	(theMap->pc).velocity.x = (theMap->pc).velocity.y = 0;
      }
      speed--;
    }
    /* Start drawing routine */
    clear_to_color(buffer, BLACK);
    for (i = 0; i < 20; i++) {
      for (j = 0; j < 15; j++) {
	blit(dungeon, buffer, theMap->tilemap[i][j] * 9, 0, i * 32, j * 32, 32, 32);
      }
    }
    for (i = 0; i < 20; i++) { // draw platforms
      for (j = 0; j < 15; j++) {
	if (theMap->platforms[i][j]) { // if there's a platform here
	  if (i == 0 || i == 19) {
	    masked_blit(platmap, buffer, 0, 32, i * 32, (j + 1) * 32 - 16, 32, 32);
	  }
	  else if (theMap->platforms[i - 1][j] && theMap->platforms[i + 1][j]) { // check if there are on either side
	    masked_blit(platmap, buffer, 0,32, i * 32, (j + 1) * 32 - 16, 32, 32); // and draw the appropriate image
	  }
	  else if (theMap->platforms[i - 1][j]) {
	    masked_blit(platmap, buffer, 32,32, i * 32, (j + 1) * 32 - 16, 32, 32);
	  }
	  else if (theMap->platforms[i + 1][j]) {
	    masked_blit(platmap, buffer, 32,0, i * 32, (j + 1) * 32 - 16, 32, 32);
	  }
	  else {
	    masked_blit(platmap, buffer, 0,0, i * 32, (j + 1) * 32 - 16, 32,32);
	  }
	}
      }
    }
    for (i = 0; i < 300; i++) { // draw crates
      if (theMap->crates[i].used) {
	switch (theMap->crates[i].damage) { // draw appropriate image based on damage to crate
	case 4:
	  blit(crateimg, buffer, 0,0, theMap->crates[i].location.x, theMap->crates[i].location.y - 16, 32,32);
	  break;
	case 3:
	  blit(crateimg, buffer, 32, 0, theMap->crates[i].location.x, theMap->crates[i].location.y - 16, 32, 32);
	  break;
	case 2:
	  blit(crateimg, buffer, 0, 32, theMap->crates[i].location.x, theMap->crates[i].location.y - 16, 32, 32);
	  break;
	case 1:
	  blit(crateimg, buffer, 32, 32, theMap->crates[i].location.x, theMap->crates[i].location.y - 16, 32, 32);
	  break;
	}
      }
    }
    draw_character_n(buffer, &(theMap->pc)); // draw the character
    draw_npc(buffer, &tnpc); // draw test NPC
    if (ex_frame <= 40 && ex_frame >= 0) { // draw the explosion
      masked_blit(explosions, buffer, (ex_frame % 8) * 32, ((int) ex_frame / 8) * 32 + (4 * 32), ex_x * 32, ex_y * 32 - 16, 32, 32);
      ex_frame++;
    }
    // show pschopyretic power indicator
    circlefill(buffer, SCREEN_W - 30, 30, 15, makecol(pyropower,0,0));
    /* start debug output (erase close-comment here to comment out code) */
    textprintf_ex(buffer, font, 5, 5, WHITE, BLACK, "Speed X: %.2f", (theMap->pc).velocity.x);
    textprintf_ex(buffer, font, 5, 13, WHITE, BLACK, "Speed Y: %.2f", (theMap->pc).velocity.y); 
    textprintf_ex(buffer, font, 5, 21, WHITE, BLACK, "numJumps: %d", numJumps);
    textprintf_ex(buffer, font, 100, 21, WHITE, BLACK, "state: %d", (theMap->pc).state);
    textprintf_ex(buffer, font, 5, 29, WHITE, BLACK, "location.x: %.2f", (theMap->pc).location.x);
    textprintf_ex(buffer, font, 5, 37, WHITE, BLACK, "location.y: %.2f", (theMap->pc).location.y);
    textprintf_ex(buffer, font, 155, 29, WHITE, BLACK, "pc.tile.x: %d", (theMap->pc).tile.x);
    textprintf_ex(buffer, font, 155, 37, WHITE, BLACK, "pc.tile.y: %d", (theMap->pc).tile.y);    
    textprintf_ex(buffer, font, 5, 45, WHITE, BLACK, "mouse_x: %d", mouse_x);
    textprintf_ex(buffer, font, 135, 45, WHITE, BLACK, "mouse_y: %d", mouse_y);
    textprintf_ex(buffer, font, 5, 53, WHITE, BLACK, "pyropower: %d", pyropower);
    textprintf_ex(buffer, font, 125, 53, WHITE, BLACK, "dbg_field: %d", dbg_field);
    /* end debug output */
    masked_blit(pointer, buffer, 0,0, mouse_x - 3, mouse_y - 3, 32, 32);
    if (key[KEY_TAB]) { // take screenshot
      save_bitmap("screenshot.bmp", buffer, NULL);
    }
    if (!isAlive) {
      textout_centre_ex(screen, font, "Game over. Press any key to continue.", SCREEN_W/2, SCREEN_H/2, WHITE, BLACK);
      clear_keybuf();
      readkey();
      break;
    }
    blit(buffer, screen, 0,0,0,0, SCREEN_W, SCREEN_H);
  }
  destroy_bitmap(buffer);
  return 0;
}
END_OF_MAIN()
