#include "pyretic.h"

/* This is probably reinventing the wheel, but I couldn't find it in the standard
   library. This just takes a char *, a stream and a length and prints the data to
   the stream to the given length, ignoring \0's.
*/
int fout(FILE *stream, const char *data, int len) {
  int i;
  for (i = 0; i < len; i++) {
    fputc((int) data[i], stream);
  }
  return 0;
}

int fin(FILE *stream, char *buf, int len) {
  int i;
  int c;
  for (i = 0; i < len && (c = fgetc(stream)) != EOF; i++) {
    buf[i] = (char) c;
  }
  return 0;
}

int init_map(map *theMap) {
  memset(theMap, 0, sizeof(map));
  return 0;
}

int load_map(map *it, const char *filename) {
  FILE *f = fopen(filename, "r");
  if (f == NULL)
    return -1;
  map_to_chars mtc;
  fin(f, mtc.c, sizeof(map));
  *it = mtc.m;
  return 0;
}

/* WARNING: This will overwrite the file without warning. Check if it exists first. */
int save_map(map *it, const char *filename) {
  FILE *f = fopen(filename, "w");
  if (f == NULL)
    return -1;
  map_to_chars mtc;
  mtc.m = *it;
  fout(f, mtc.c, sizeof(map));
  return 0;
}

int damage_crate(map *it, int cindex, int amt) {
  crate *c;
  c = lookup_crate_number(it, cindex);
  if (c == NULL)
    return -1;
  if (c->damage > 0)
    c->damage -= amt;
  if (c->damage <= 0)
    destroy_crate(it, cindex);
  return 0;
}

int create_crate(map *it, int x, int y) {
  it->obj_map[x][y] = 1;
  int i;
  for (i = 0; i < 300; i++) {
    if (it->crates[i].used == 0) {
      break;
    }
  }
  if (i == 300 && it->crates[i].used != 0)
    return -1;
  it->crates[i].tile.x = x;
  it->crates[i].tile.y = y;
  it->crates[i].state = 1;
  it->crates[i].number = i;
  it->crates[i].location.x = it->crates[i].tile.x * 32;
  it->crates[i].location.y = it->crates[i].tile.y * 32;
  it->crates[i].damage = 4;
  it->crates[i].used = 1;
  return 0;
}

int destroy_crate(map *it, int cindex) {
  crate *c = lookup_crate_number(it, cindex);
  if (c == NULL) {
    return -1;
  }
  it->obj_map[c->tile.x][c->tile.y] = 0;
  it->crates[c->number].used = 0;
  memset(c, 0, sizeof(crate));
  return 0;
}

int update_crate(map *it, int cindex) {
  crate *c = lookup_crate_number(it, cindex);
  if (c == NULL)
    return -1;
  it->obj_map[c->tile.x][c->tile.y] = 0; // delete it at old position
  if (c->state) {
    c->velocity.y += 0.5;
  }
  c->location.x += c->velocity.x;
  c->location.y += c->velocity.y;
  c->tile.x = (int) floor(c->location.x / 32);
  c->tile.y = (int) floor(c->location.y / 32);  
  if ((it->platforms[c->tile.x][c->tile.y] || 
       it->obj_map[c->tile.x][c->tile.y + 1] == 1) && c->state) {
    c->state = 0;
    c->location.y = c->tile.y * 32;
  }
  if (!(it->platforms[c->tile.x][c->tile.y] || it->obj_map[c->tile.x][c->tile.y + 1] == 1)) {
    c->state = 1;
  }
  if (!c->state) {
    c->velocity.y = 0;
  }
  if (c->location.y >= SCREEN_H + 40)
    destroy_crate(it, cindex);
  if (c->damage == 0) {
    destroy_crate(it, cindex);
  }
  it->obj_map[c->tile.x][c->tile.y] = 1; // create it at new position
  return 0;
}

int lookup_crate(map *it, int x, int y) {
  int i;
  for (i = 0; i < 300; i++) 
    if (it->crates[i].tile.x == x && it->crates[i].tile.y == y)
      return i;
  return -1;
}

crate *lookup_crate_number(map *it, int num) {
  int i = 0;
  if (it->crates[i].used == 0)
    return NULL;
  return &(it->crates[i]);
}
