#include "pyretic.h"

void fill_vector(double rads, double length, vector *target) {
  target->radians = rads;
  target->len = length;
}

void fill_cartesian(double x, double y, cartesian *target) {
  target->x = x;
  target->y = y;
}

/* This changes the passed-in cartesian * in place, but also returns the pointer.
   Should I do that, or copy the data, or ... ? */
cartesian *vec_to_cart(cartesian *cartOut, vector *vectorIn) {
  cartOut->x = vectorIn->len * cos(vectorIn->radians);
  cartOut->y = vectorIn->len * sin(vectorIn->radians);
  return cartOut;
}

/* See note above for vec_to_cart. */
vector *cart_to_vec(vector *vectorOut, cartesian *cartIn) {
  vectorOut->len = sqrt(cartIn->x * cartIn->x + cartIn->y * cartIn->y);
  vectorOut->radians = atan2(cartIn->y, cartIn->x);
  return vectorOut;
}

/* Cartesian values are easy to add, because you just add the x and y values
   separately. To add vectors I convert them to cartesians. 
   Should the return value be void here? That's all I can think to do. */
void vector_add(vector *vec1, vector *vec2) {
  cartesian cart1, cart2;
  vec_to_cart(&cart1, vec1);
  vec_to_cart(&cart2, vec2);
  cart1.x += cart2.x;
  cart1.y += cart2.y;
  cart_to_vec(vec1, &cart1);
}

/* See note above. */
void vector_negate(vector *vec) {
  cartesian cart;
  vec_to_cart(&cart, vec);
  cart.x = 0 - cart.x;
  cart.y = 0 - cart.y;
  cart_to_vec(vec, &cart);
}

void vector_bounce(vector *vec, double angle) {
  double newRads;
  newRads = vec->radians - angle;
  newRads = 0 - newRads;
  newRads += angle;
  vec->radians = newRads;
}

void bounce_with_friction(vector *vec, double angle) {
  vector newvec = *vec;
  vector_bounce(&newvec, angle); /* can't just call it with vec because */
  newvec.len -= 0.6;		 /* that would do the edits in calling context */
  *vec = newvec; 		 /* before I'm ready */
}
